@@ -21,8 +21,7 @@
 """
 
 from MoinMoin.Page import Page
-from MoinMoin import wikiutil, wikiaction, config
-from MoinMoin.util import MoinMoinNoFooter
+from MoinMoin import wikiutil, config
 import os, errno, tempfile, time, zipfile, codecs, shutil
 
 
@@ -99,22 +98,12 @@ def execute(pagename, request):
     info.compress_type = zipfile.ZIP_STORED     # ZIP_STORED = uncompressed
     zipper.writestr(info, "application/vnd.oasis.opendocument.text")
 
-    # Set HTTP Header: mimetype and filename
-    # This has to come before any request writing. Even request.redirectedOutput is a request writing! 
-    # The webserver will have no clue what to serve if headers are missing. 
-    # Note that RuntimeErrors create their one headers and pages and thus should come before this: Now's too late
-    # for HTML-Error pages.
-    request.http_headers([
-      'Content-Type: application/vnd.oasis.opendocument.text',
-      'Content-Disposition: attachment; filename="%s"' % (filenameASCII)
-    ])
-
     # write inner file content.xml by calling the util formatter (returns unicode which has to be encoded as utf-8)
     formatter = Formatter(request, store_pagelinks=1)
     page = Page(request, pagename, formatter = formatter)
     
     (attachments, contentString) = \
-        _filter_content( request.redirectedOutput(page.send_page, request, count_hit=0).encode( 'utf-8' ) )
+        _filter_content( request.redirectedOutput(page.send_page, request, emit_headers=0, count_hit=0).encode( 'utf-8' ) )
     
     info = zipfile.ZipInfo("content.xml", now)
     info.compress_type = zipfile.ZIP_DEFLATED     # ZIP_DEFLATED = compressed
@@ -611,11 +600,18 @@ xmlns:xlink="http://www.w3.org/1999/xlink"/>"""
     # close ZIP file
     zipper.close()
 
+
+    # Set HTTP Header: mimetype and filename
+    #
+    # Note that RuntimeErrors create their one headers and pages and thus should come before this: Now's too late
+    # for HTML-Error pages.
+    request.emit_http_headers([
+      'Content-Type: application/vnd.oasis.opendocument.text',
+      'Content-Disposition: attachment; filename="%s"' % (filenameASCII)
+    ])
+
     # send data (without 'rb' the windows version doesn't work at all!)
     shutil.copyfileobj( open( pathToZIP, 'rb' ), request, 8192 )
 
     # remove temporary data
     shutil.rmtree(outputdir)
-    
-    # suppress ...</body></html>
-    raise MoinMoinNoFooter
