@@ -24,7 +24,7 @@
 
 import sys, re
 
-from MoinMoin.formatter.base import FormatterBase # for 1.6: from MoinMoin.formatter import FormatterBase
+from MoinMoin.formatter import FormatterBase
 from MoinMoin.action import AttachFile
 from MoinMoin.Page import Page
 from MoinMoin import wikiutil
@@ -59,7 +59,9 @@ validChildren = {'office:document-content': ['office:body', 'office:automatic-st
 validGrandChildren = { 'table:table-cell': { E_TEXT: 'text:p' }, \
                                          'text:list-item': { E_TEXT: 'text:p' }}
 
-whitespaceSearch = re.compile('(  +)')  # find two or more space characters
+# find two or more space characters or a single space character if at the beginning (useful for preformatted 
+# paragraphs where a first single space character is always ignored)
+whitespaceSearch = re.compile('(  +|^ )')  
 
 
 
@@ -528,7 +530,9 @@ class Formatter(FormatterBase):
         
         
     def linebreak(self, preformatted = 1, **kargs): 
-        self.Node.addChild( Node("text:line-break") )
+        # Suppress starting line break
+        if self.Node.children:
+            self.Node.addChild( Node("text:line-break") )
         return ""
 
 
@@ -574,26 +578,6 @@ class Formatter(FormatterBase):
             newNode.setAttribute("text:style-name","Preformatted_20_Text")
         else:
             pNode = self.close("text:p", "preformatted_function") # Mark text:p as other might use it too
-            if pNode <> None:
-                # Remove unnecessary linebreak at begin of paragraph
-                if len(pNode.children) > 0:
-                    firstChild = pNode.children[ 0 ]
-                    if firstChild.element == 'text:line-break':
-                        del pNode.children[ 0 ]
-                # Remove unnecessary linebreak at end of paragraph
-                if len(pNode.children) > 0:
-                    lastChild = pNode.children[ -1 ]
-                    if lastChild.element == 'text:line-break':
-                        del pNode.children[ -1 ]
-                # if paragraph starts with a single space character it's always ignored in OpenDocument
-                # (No idea what would happen with multiple space characters as we already converted them in self.text())
-                if len( pNode.children ) > 0 and pNode.children[ 0 ].element == E_TEXT:
-                    textChild = pNode.children[ 0 ]
-                    if textChild.content.startswith(' '):
-                        textChild.content = textChild.content[ 1: ]
-                        newNode = Node("text:s")
-                        newNode.setAttribute("text:c", "1")
-                        pNode.insertChild( 0, newNode )                
         return ""
 
 
@@ -703,21 +687,45 @@ class Formatter(FormatterBase):
     # Create Text, but replace long whitespace by whitespace tag
     def text(self, text, **kargs):
 
-        textlist = whitespaceSearch.split(text)   # split text by two or more space characters
+        # split text by two or more space characters (or if a single space is at the beginning of a text element)
+        textlist = whitespaceSearch.split(text)   
         whitespace = False   # split never returns the delimiter as first token
         
         for token in textlist:
             if whitespace:
                 newNode = Node("text:s")
-                newNode.setAttribute("text:c",str( len(token) ))
+                if len(token) > 1:
+                    newNode.setAttribute("text:c",str( len(token) ))
                 self.Node.addChild( newNode ) 
             else:
-                self.Node.addTextChild( token )
+                # Add Text. Split preformatted text into its lines
+                if self.Node.mark == 'preformatted_function':
+                    lines = token.split('\n')
+                    for i in range(0, len(lines)):
+                        line = lines[i]
+                        if len(line) > 0:
+                            self.Node.addTextChild( line )
+                        elif i == 0:
+                            continue                     # don't start with a linebreak
+                        if (i < len(lines) - 1):       # don't end with a linebreak
+                            self.linebreak();
+                else:
+                    self.Node.addTextChild( token )
             whitespace = whitespace != True    # toggle whitespace 
             
         return ""
 
 
+    def transclusion(self, on, **kargs):
+        # TODO: Embed another object
+        return ''
+
+
+    def transclusion_param(self, **kargs):
+        # TODO: Give parameters to embedded object
+        return ''
+
+
     def underline(self, on, **kargs): 
         self.formatTag( on, "underline" )
         return ""
